<?php
function tkswim_fehlerbehandlung($nr, $text, $datei, $zeile){
	if(ini_get('error_reporting') == 0){
		return;
	}
	global $_tmpl, $_db;

	# Template-Dateien melden ueberfluessigerweise Fehler -> ignorieren
	if(strpos(str_replace('\\','/',$datei),PFAD_TEMPLATESC)!==false){
		return;
	}

	# Type der Meldung bestimmen
	$type = 'php';
	if(in_array($nr,array(E_USER_ERROR,E_USER_WARNING,E_USER_NOTICE))){ # benutzerdefinierter Fehler
		if(substr($text,0,8) == '[tkswim]'){ # Fehlermeldung von tkswim
			$type = 'tkswim';
			$text = substr($text,8);
		}
		elseif(substr($text,0,4) == '[db]'){ # Fehlermeldung der Datenbank
			$type = 'db';
			$text = substr($text,5);
		}
		elseif(substr($text,0,7) == '[login]'){ # Loginmeldung
			$type = 'login';
			$text = substr($text,8);
		}
		elseif(substr($text,0,13) == 'Smarty error:'){ # Meldung vom Smarty
			$type = 'smarty';
			$text = substr($text,14);
		}
	}

	# Fehlerstufe bestimmen
	$fehlerstufe = 'Notice';
	switch ($nr){
	case E_USER_ERROR:
		$fehlerstufe = 'Error';
		break;
	case E_USER_WARNING:
	case E_WARNING:
		$fehlerstufe = 'Warning';
		break;
	}

	# Array mit den zu speichernden Elementen fuellen
	$eintrag = array(
		'art'       => $type,
		'stufe'     => $fehlerstufe,
		'datei'     => $datei,
		'zeile'     => $zeile,
		'zeit'      => time(),
		'meldung'   => $text,
		'sonstiges' => ''
	);

	# bei den Typen login, smarty und db noch einige Aktionen vornehmen
	switch($type){
	case 'login':
		# uebergebene Meldung auseinanderpfrimeln
		list($eintrag['meldung'],$eintrag['sonstiges']) = explode('|',$text,2);
		break;
	case 'smarty':
		# richtige Datei suchen
		$debug = debug_backtrace();
		if(is_array($debug)){
			$debug = array_pop($debug);
			$eintrag['datei'] = $debug['file'];
			$eintrag['zeile'] = $debug['line'];
		}
		break;
	case 'db':
		# uebergebenen Inhalt (Datei, Zeile, Fehlermeldung und Query) auseinanderpfrimeln und in $eintrag stecken
		list($datei,$zeile,$fehlermeldung,$query) = explode(';',$text,4);
		$debug_original = debug_backtrace();
		if(is_array($debug_original)){
			$debug = array_pop($debug_original);
			if(isset($debug['function']) and $debug['function'] == 'require_once'){
				$debug = array_pop($debug_original);
			}
			$datei = $debug['file'];
			$zeile = $debug['line'];
		}
		if(substr($query,0,1) == '[' and substr($query,-1,1) == ']'){ # kein Query
			$text = substr($query,1,strlen($query)-2);
			if(!empty($fehlermeldung)){
				$text .= ' ('.$fehlermeldung.')';
			}
			$query = '-';
		}
		else{
			$text = $fehlermeldung;
		}
		$eintrag['datei'] = $datei;
		$eintrag['zeile'] = $zeile;
		$eintrag['meldung'] = $text;
		$eintrag['sonstiges'] = $query;
		break;
	default:
		break;
	}

	# Meldung in Log-Datei schreiben
	foreach($eintrag as $key => $value){
		$value = str_replace('¡','|',$value); # Trennzeichen entschärfen
		$eintrag[$key] = preg_replace("/(\015\012)|(\015)|(\012)/",' ',$value); # Zeilenumbrueche entfernen
	}
	$neuezeile = implode('¡',$eintrag)."\n";
	$fh = fopen(PFAD_LOGDATEIEN.'logdatei.log','a');
	fwrite($fh,$neuezeile,strlen($neuezeile));
	fclose($fh);

	# fuer die einzelnen Fehlertypen das Verhalten bei Auftreten eines Fehlers festlegen
	switch($eintrag['art']){
	case 'tkswim':
		switch ($nr){
		case E_USER_ERROR:
			$_tmpl->assign('fehler','Es ist ein Fehler aufgetreten.');
			$text = 'tkswim hat einen schwerwiegenden Fehler gemeldet, das Generieren der Seite wurde abgebrochen.
					Der Grund für den Fehler wurde gespeichert und wird baldmöglichst behoben.';
			$_tmpl->assign('inhalt',$text);
			global $_db;
			$_db->speichereGeaenderteTabellen();
			$inhalt = $_tmpl->fetch('tmpl.inc.html');
			if(isset($_einst) and $_einst['globaleeinstellungen']['inhaltkomprimiertausliefern']){
				ob_start('ob_gzhandler');
				echo $inhalt;
				ob_end_flush();
			}
			else{
				echo $inhalt;
			}
			exit;
		case E_USER_WARNING:
			if(TKSWIM_DEBUG){
				$_tmpl->assign('fehler','[Warnung] [tkswim] es ist ein Fehler aufgetreten (nähere Informationen: siehe Log-Datei).');
			}
			else{
				$_tmpl->assign('fehler','[Warnung] Es ist ein Fehler aufgetreten, möglicherweise bringt die Seite unerwartete Ergebnisse.');
			}
			break;
		case E_USER_NOTICE:
			if(TKSWIM_DEBUG){
				$_tmpl->assign('fehler','[Hinweis] [tkswim] Es ist ein Fehler aufgetreten (nähere Informationen: siehe Log-Datei).');
			}
			break;
		}
		break;
	case 'db':
		switch ($nr){
		case E_USER_ERROR:
			$_tmpl->assign('fehler','Es ist ein Datenbankfehler aufgetreten.');
			if(TKSWIM_DEBUG){
				$text = '<h3>Debuginformationen</h3>';
				$text .= '<h4>Datei:</h4><p>'.$eintrag['datei'].' (Zeile: '.$eintrag['zeile'].')</p>';
				$text .= '<h4>Fehlermeldung:</h4><p>'.$eintrag['meldung'].'</p>';
				$text .= '<h4>Query:</h4><p>'.str_replace('[br]','<br />',$eintrag['sonstiges']).'</p>';
			}
			else{
				$text = 'Beim Zugriff auf die Datenbank ist ein Fehler aufgetreten, das Generieren der Seite wurde abgebrochen.
						Der Grund für den Fehler wurde gespeichert und wird baldmöglichst behoben.';
			}
			$_tmpl->assign('inhalt',$text);
			global $_db;
			$_db->speichereGeaenderteTabellen();
			$inhalt = $_tmpl->fetch('tmpl.inc.html');
			if(isset($_einst) and $_einst['globaleeinstellungen']['inhaltkomprimiertausliefern']){
				ob_start('ob_gzhandler');
				echo $inhalt;
				ob_end_flush();
			}
			else{
				echo $inhalt;
			}
			exit;
		case E_USER_WARNING:
			if(TKSWIM_DEBUG){
				$_tmpl->assign('fehler','[Warnung] Es ist ein Datenbankfehler aufgetreten (nähere Informationen: siehe Log-Datei).');
			}
			else{
				$_tmpl->assign('fehler','[Warnung] Es ist ein Datenbankfehler aufgetreten, möglicherweise bringt die Seite unerwartete Ergebnisse.');
			}
			break;
		case E_USER_NOTICE:
			if(TKSWIM_DEBUG){
				$_tmpl->assign('fehler','[Hinweis] Es ist ein Datenbankfehler aufgetreten (nähere Informationen: siehe Log-Datei).');
			}
			break;
		}
		break;
	case 'login':
		# keine weitere Ausgabe, das macht die login-Datei selbst
		break;
	case 'smarty':
		switch ($nr){
		case E_USER_ERROR:
			$_tmpl->assign('fehler','Es ist ein Fehler aufgetreten.');
			$text = 'Smarty hat einen schwerwiegenden Fehler gemeldet, das Generieren der Seite wurde abgebrochen.
					Der Grund für den Fehler wurde gespeichert und wird baldmöglichst behoben.';
			$_tmpl->assign('inhalt',$text);
			global $_db;
			$_db->speichereGeaenderteTabellen();
			$inhalt = $_tmpl->fetch('tmpl.inc.html');
			if(isset($_einst) and $_einst['globaleeinstellungen']['inhaltkomprimiertausliefern']){
				ob_start('ob_gzhandler');
				echo $inhalt;
				ob_end_flush();
			}
			else{
				echo $inhalt;
			}
			exit;
		case E_USER_WARNING:
			if(TKSWIM_DEBUG){
				$_tmpl->assign('fehler','[Warnung] [smarty] es ist ein Fehler aufgetreten (nähere Informationen: siehe Log-Datei).');
			}
			else{
				$_tmpl->assign('fehler','[Warnung] Es ist ein Fehler aufgetreten, möglicherweise bringt die Seite unerwartete Ergebnisse.');
			}
			break;
		case E_USER_NOTICE:
			if(TKSWIM_DEBUG){
				$_tmpl->assign('fehler','[Hinweis] [smarty] Es ist ein Fehler aufgetreten (nähere Informationen: siehe Log-Datei).');
			}
			break;
		}
		break;
	case 'php':
		switch ($nr){
		case E_WARNING:
			if(TKSWIM_DEBUG){
				$_tmpl->assign('fehler','[Warnung] [php] es ist ein Fehler aufgetreten (nähere Informationen: siehe Log-Datei).');
			}
			else{
				$_tmpl->assign('fehler','[Warnung] Es ist ein Fehler aufgetreten, möglicherweise bringt die Seite unerwartete Ergebnisse.');
			}
			break;
		case E_NOTICE:
			if(TKSWIM_DEBUG){
				$_tmpl->assign('fehler','[Hinweis] [php] Es ist ein Fehler aufgetreten (nähere Informationen: siehe Log-Datei).');
			}
			break;
		}
		break;
	default:
		break;
	}

} # Ende der Funktion, EOF
?>